/*
 * Copyright (c) 2005 DocJava, Inc. All Rights Reserved.
 */
package classUtils.putils;

import classUtils.loaders.ByteCodeContainer;
import classUtils.pathUtils.PathTabbedPane;
import futils.DirList;
import futils.Futil;
import futils.Ls;
import gui.In;
import utils.StringUtils;
import utils.SystemUtils;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.util.Arrays;
import java.util.Comparator;
import java.util.StringTokenizer;
import java.util.jar.JarFile;
import java.util.jar.JarInputStream;
import java.util.prefs.Preferences;
import java.util.zip.ZipEntry;

public class LibraryPathUtils {

    private static String key = "libraryPath";

    public static void main(String[] args) {
        printLibraryPaths();
    }

    public static void printLibraryPaths() {
        System.out.println(
                "The getLibraryPath=");
        String s[] = getLibraryPaths();
        print(s);
    }


    public static boolean contains(
            String s1,
            String s2) {
        if (-1 == s1.indexOf(s2)) return false;
        return true;
    }



    public static void print(Object o) {
        System.out.println(o);
    }




    /**
     * takes a string of the java/lang/String.class form and returns
     * java.lang.String
     */
    public static String makeClassString(String s) {
        s = StringUtils.sub(s, ".class", "");
        s = s.replace('/', '.');
        s = s.replace('\\', '.');
        //s = ReplaceString.sub(s, "c:.lyon.j4p.classes.", "");
        return s;
    }


    public static void print(Object s[]) {
        if (s == null) return;
        for (int i = 0; i < s.length; i++)
            System.out.println(s[i]);
        System.out.println("found:" + s.length + " items");
    }


    public static void print(Class s[]) {
        if (s == null) return;
        for (int i = 0; i < s.length; i++)
            System.out.println(s[i].getName());
        System.out.println("found:" + s.length + " Classes");
    }


    public static int countJars(String s[]) {
        int n = 0;
        for (int i = 0; i < s.length; i++)
            if (s[i].endsWith("jar")) n++;
        return n;
    }

    public static JarFile getAJarFile() {
        File f =
                Futil.getReadFile("select a jar file");
        JarFile jf = null;
        try {
            jf = new JarFile(f);
        } catch (IOException e) {
            e.printStackTrace();
        }
        return jf;
    }

    public static JarFile[] getJarFiles() {
        DirList dl = new DirList(".jar");
        File f[] = dl.getFiles();
        JarFile jf[] = new JarFile[f.length];
        for (int i = 0; i < jf.length; i++)
            try {
                jf[i] = new JarFile(f[i]);
            } catch (IOException e) {
                System.out.println(
                        "could not make jar file out of:" +
                        f[i]);
            }
        return jf;
    }

    public static String[] getLibraryPaths() {
        String s = SystemUtils.getLibraryPath();
        return parsePath(s);
    }

    /**
     * Input a path separator delimited set of directories and return one
     * per string element in an array.
     */
    private static String[] parsePath(String classPath) {
        String pathSeparator = System.getProperty("path.separator");
        StringTokenizer st =
                new StringTokenizer(
                        classPath,
                        pathSeparator);
        int n = st.countTokens();
        String s[] = new String[n];
        for (int i = 0; i < n; i++)
            s[i] = st.nextToken();
        return s;
    }


    private static Package[] getPackages() {
        return Package.getPackages();
    }

    public static void print(Package[] pkgs) {
        SortPackages(pkgs);
        for (int i = 0; i < pkgs.length; i++)
            System.out.println("  " + pkgs[i].getName());
        System.out.println(
                "printed:" +
                pkgs.length +
                " packages.");
    }

    private static void SortPackages(Package[] pkgs) {
        Arrays.sort(
                pkgs,
                new PackageComparator());
    }

    public static void dumpAClassFile() {
        FileInputStream fis = Futil.getFileInputStream(
                "Select a class file");
        getComputableObjectImplementor(fis);
    }

    public static void dumpClassFile(File f[]) {
        for (int i = 0; i < f.length; i++)
            dumpClassFile(f[i]);
    }

    public static void dumpClassFile(File f) {
        try {
            FileInputStream fis = new FileInputStream(f);
            Class c = getComputableObjectImplementor(fis);
            if (c != null)
                System.out.println(
                        "This class is a ComputableObject" +
                        c.getName());
            fis.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static Class getComputableObjectImplementor(
            FileInputStream fis) {
        classUtils.dumper.ByteCodeContainer bcc =
                classUtils.dumper.ByteCodeContainer.getByteCodeContainer(
                        fis);
        bcc.loadIt();
        Class c = bcc.getClass();
        Class interfaces[] = c.getInterfaces();
        for (int i = 0; i < interfaces.length; i++)
            if (interfaces[i].getName().endsWith("ComputableObject"))
                return c;
        return null;
    }

    public static void dumpAllFilesInADirectory() {
        File f[] = Ls.getWildFiles(
                Futil.getReadDirFileJTree("select a class directory"),
                "class");
        dumpClassFile(f);
    }


    public static void promptUserForClassName()
            throws ClassNotFoundException,
                   IllegalAccessException,
                   InstantiationException {
        String cn = In.getString("enter a class name");
        Class c = Class.forName(cn);
        In.message(c.newInstance().toString());
    }

    public static Class getClass(String className) {
        try {
            return Class.forName(className);
        } catch (ClassNotFoundException e) {
            In.message("in Command getClass:" + e);
        }
        return null;
    }

    public static String[] splitClassName(String clsName) {
        int i = clsName.lastIndexOf('.');
        String[] result = new String[2];
        if (i == -1) {
            result[0] = "";
            result[1] = clsName;
        } else {
            result[0] = clsName.substring(0, i);
            result[1] = clsName.substring(i + 1);
        }
        return result;
    }

    public static String[] getStringArray(String classes) {
        StringTokenizer st = new StringTokenizer(classes, ";, ");
        String[] array = new String[st.countTokens()];
        int i = 0;
        while (st.hasMoreTokens())
            array[i++] = st.nextToken();
        return array;
    }

    public static void saveClassPath() {
        Preferences p = Preferences.userRoot();
        p.put(key, SystemUtils.getLibraryPath());
    }

    public static void fixClassPath() {
        PathTabbedPane.fixClassPath();
    }

    public static void restoreClassPath() {
        Preferences p = Preferences.userRoot();
        setClassPath(p.get(key, SystemUtils.getLibraryPath()));
    }


    private static class PackageComparator
            implements Comparator {
        public int compare(Object a, Object b) {
            return ((Package) a).getName()
                    .compareTo(((Package) b).getName());
        }
    }

    static void loadJarEntry(JarFile jf) {
        try {
            ZipEntry ze = jf.getEntry(null);
            jf.getInputStream(ze);
            JarInputStream in = new JarInputStream(
                    new FileInputStream(""));
            //ZipEntry ze=  in.getNextEntry();
            System.out.println("nextEntry is:" + ze);
            byte b[] = new byte[(int) ze.getSize()];
            in.read(b);
            String s = ze.getName();
            if (!s.endsWith(".class")) return;
            System.out.println("loading:" + s);
            ByteCodeContainer bcc = new ByteCodeContainer(b);
            bcc.loadIt();
            in.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public static void setClassPath(String s) {
        System.setProperty("java.class.path", s);
    }
}
